from math import sqrt

class Vector3(object):
	"""Dummy Vector3 class if minieigen is not found"""
	def __init__(self,*args):
		if len(args)==1:
			a = args[0]
			if isinstance(a,(tuple,list)) and len(a)==3 or isinstance(a,Vector3):
				self.x = float(a[0])
				self.y = float(a[1])
				self.z = float(a[2])
			else:
				raise TypeError, "%s not valid argument for Vector3"%str(a)
		elif len(args)==3:
			self.x = float(args[0])
			self.y = float(args[1])
			self.z = float(args[2])
		else:
			raise TypeError, "%s not valid arguments for Vector3"%str(args)
	def cross(self,vec):
		"""Returns cross product of receiver and given vector

		:param Vector3 vec: given vector
		:rtype: Vector3
		"""
		return Vector3(self.y*vec.z-self.z*vec.y,self.z*vec.x-self.x*vec.z,self.x*vec.y-self.y*vec.x)
	def dot(self,vec):
		"""Returns dot product of receiver and given vector

		:param Vector3 vec: given vector
		:rtype: float
		"""
		return self.x*vec.x+self.y*vec.y+self.z*vec.z
	def maxAbsCoeff(self):
		"""Returns maximum absolute value over all elements

		:rtype: float
		"""
		return max(abs(self[i]) for i in xrange(3))
	def norm(self):
		"""Returns Euclidean norm of receiver

		:rtype: float
		"""
		return sqrt(self.squaredNorm())
	def normalize(self):
		"""Normalizes receiver"""
		n = self.norm()
		self.x /= n
		self.y /= n
		self.z /= n
	def normalized(self):
		"""Returns normalized copy of receiver
		
		:rtype: Vector3
		"""
		ret = Vector3(self.x,self.y,self.z)
		ret.normalize()
		return ret
	def squaredNorm(self):
		"""Returns square of receiver's norm

		:rtype: float
		"""
		return self.dot(self)
	def sum(self):
		"""Returns sum of receiver's elements

		:rtype: float
		"""
		return sum(self[i] for i in xrange(3))
	def __abs__(self):
		return self.norm()
	def __add__(self,vec):
		return Vector3(self.x+vec.x,self.y+vec.y,self.z+vec.z)
	def __div__(self,v):
		return self*(1./v)
	def __eq__(self,v):
		return all(self[i]==v[i] for i in xrange(3))
	def __getitem__(self,i):
		if not i in (0,1,2):
			raise IndexError, "Index %s out of range 0..2"%str(i)
		return self.x if i==0 else self.y if i==1 else self.z if i==2 else None
	def __iadd__(self,vec):
		return self+vec
	def __idiv__(self,v):
		return self/v
	def __imul__(self,v):
		return self*v
	def __isub__(self,vec):
		return self-vec
	def __len__(self):
		return 3
	def __mul__(self,f):
		return Vector3(self.x*f,self.y*f,self.z*f)
	def __ne__(self,v):
		return not self==v
	def __neg__(self):
		return Vector3(-self.x,-self.y,-self.z)
	def __repr__(self):
		return str(self)
	def __rmul__(self,f):
		return self*f
	def __setitem__(self,i,v):
		if not i in (0,1,2):
			raise IndexError, "Index %s out of range 0..2"%str(i)
		if   i==0: self.x = v
		elif i==1: self.y = v
		elif i==2: self.z = v
	def __str__(self):
		return 'Vector3(%e,%e,%e)'%(self.x,self.y,self.z)
	def __sub__(self,vec):
		return Vector3(self.x-vec.x,self.y-vec.y,self.z-vec.z)
